/* Unit test suite for LOGFONT objects
 *
 * Copyright 2002 Mike McCormack
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "wine/test.h"
#include "winbase.h"
#include "wingdi.h"

#define STRUCTOFFSET(type,field) ((int)&((type *)0)->field)

START_TEST(gdiobj)
{
    LOGFONTA lf, lfout;
    HFONT hfont;

    memset(&lf, 0, sizeof lf);

    lf.lfCharSet = ANSI_CHARSET;
    lf.lfClipPrecision = CLIP_DEFAULT_PRECIS;
    lf.lfEscapement = 0;
    lf.lfOrientation = 0;
    lf.lfWeight = FW_DONTCARE;
    lf.lfHeight = 16;
    lf.lfWidth = 16;
    lstrcpyA(lf.lfFaceName,"Arial");
    lf.lfQuality = DEFAULT_QUALITY;

    hfont = CreateFontIndirectA(&lf);
    ok(hfont != 0, "CreateFontIndirect failed");

    ok(GetObjectA(hfont, sizeof lfout, &lfout) == sizeof(lfout),
       "GetObject returned wrong size");

    ok(!memcmp(&lfout, &lf, STRUCTOFFSET(LOGFONTA, lfFaceName)), "fonts don't match");
    ok(!lstrcmpA(lfout.lfFaceName, lf.lfFaceName), "font names don't match");

    DeleteObject(hfont);

    memset(&lf,0x7f,sizeof lf);
    hfont = CreateFontIndirectA(&lf);
    ok(hfont != 0, "CreateFontIndirectA with strange LOGFONT failed");

    ok(GetObjectA(hfont, sizeof(lfout), NULL) == sizeof(lfout),
       "GetObjectA with NULL failed");

    ok(GetObjectA(hfont, sizeof(lfout), &lfout) == sizeof(lfout),
       "GetObjectA failed");
    ok(!memcmp(&lfout, &lf, STRUCTOFFSET(LOGFONTA, lfFaceName)), "fonts don't match");
    lf.lfFaceName[sizeof(lf.lfFaceName) - 1] = 0;
    ok(!lstrcmpA(lfout.lfFaceName, lf.lfFaceName), "font names don't match");

    DeleteObject(hfont);
}


