/* Wine Profiler
 *
 * Copyright 2000-2003 CodeWeavers, Charles Loep
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#define __USE_GNU
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <dlfcn.h>

#include "profiler.h"


#ifndef RTLD_NEXT
#define RTLD_NEXT ((void *)-1L)
#endif


#ifdef DEBUG
#define db_printf(args...) printf(args)
#else
#define db_printf(args...)
#endif


#define WPROF_DETECT_EXIT


#ifdef WPROF_DETECT_EXIT

static int wrappers_initialized = 0;
static void *(*real_dlopen)(const char *filename, int mode) = 0;
static int (*real_dlclose)(void *handle) = 0;
static void (*real_exit)(int status) __attribute__((__noreturn__)) = 0;
static void (*real__exit)(int status) = 0;



static void init_wrappers()
{
	real_dlopen = dlsym(RTLD_NEXT, "dlopen");
	real_dlclose = dlsym(RTLD_NEXT, "dlclose");
	real_exit = dlsym(RTLD_NEXT, "exit");
	real__exit = dlsym(RTLD_NEXT, "_exit");
}


void *dlopen(const char *filename, int flag)
{
	if(!wrappers_initialized)
		init_wrappers();

        db_printf("[%d] dlopen(%s, %d)\n", getpid(), filename, flag);
        return real_dlopen(filename, flag);
}

int dlclose(void *handle)
{
	if(!wrappers_initialized)
		init_wrappers();
	
	db_printf("[%d] dlclose(%p)\n", getpid(), handle);
	return real_dlclose(handle);
}

void exit(int status)
{
	if(!wrappers_initialized)
		init_wrappers();

	db_printf("[%d] exit(%d)\n", getpid(), status);
	wprof_exit_thread(status);

	real_exit(status);
}

void _exit(int status)
{
	if(!wrappers_initialized)
		init_wrappers();

	db_printf("[%d] _exit(%d)\n", getpid(), status);
	wprof_exit_thread(status);

	real__exit(status);
}

#endif /*WPROF_DETECT_EXIT*/
